C To calculate the 23 equivalent axis-angle pairs.
C 
C The orientation relation between two cubic crystals can be
C   described in terms of an axis-angle pair [P1,P2,P3] and THETA,
C   or alternatively, using a rotation matrix R.
C   However, cubic crystals have a great deal of symmetry so that
C   there are in general 24 crystallographic descriptions of the
C   same orientation relationship. This subroutine the 23 alternative
C   descriptions from a give axis-angle pair. The axis is described
C   with direction the direction cosines [P1 P2 P3] and a right-handed
C   angle of rotation THETA which is input in degrees.
C 
C The matrix BEC contains 24 matrices (3x3), each describing
C   a rotation matrix which is a symmetry operation on the cubic
C   lattice.
C
C Reference
C  Worked Examples in the Geometry of Crystals
C  by H. K. D. H. Bhadeshia
C  Published by the Institute of Materials, London, 1987.
C    see pages 18 to 21, 79 (example 25).
C
      SUBROUTINE MAP_CRYSTAL_ROTAT(P1,P2,P3,THETA,BEC,Q1,Q2,Q3,THET,PI)
      IMPLICIT NONE
      REAL P1,P2,P3,THETA,BEC(24,9),Q1(23),Q2(23),Q3(23),THET(23),A5,
     &SYM(9),ANS(9),R(9),PI
      INTEGER I1,I2
C
C ensure that the components P1 P2 P3 are direction cosines
          CALL MAP_UTIL_NORM(P1,P2,P3)
C Convert THETA into radians
          THETA=THETA*PI/180.0
C
C Calculate the the rotation matrix for the given axis-angle pair
      A5=1.0
      CALL MAP_UTIL_ROT(P1,P2,P3,THETA,R)
C
C Calculate the 23 equivalent descriptions
C
      DO 1 I1=1,23
C
C Set SYM to be one of the 24 symmetry operations
           DO 2 I2=1,9
             SYM(I2)=BEC(I1+1,I2)
2         CONTINUE
C
C Generate the new rotation matrix
      CALL MAP_UTIL_PROD(ANS,SYM,R)
C
C Ensure that the new rotation matrix has columns/rows which are
C   unit vectors
        CALL MAP_UTIL_NORM(ANS(1),ANS(2),ANS(3))
        CALL MAP_UTIL_NORM(ANS(4),ANS(5),ANS(6))
        CALL MAP_UTIL_NORM(ANS(7),ANS(8),ANS(9))
C
C Calculate the new rotation angle and rotation axis
C  The method used here does not work for symmetrical R
      THET(I1)=ANS(1)+ANS(5)+ANS(9)
        IF (THET(I1) .LT. -0.999999)  GOTO 999
          Q1(I1)=ANS(8)-ANS(6)
          Q2(I1)=ANS(3)-ANS(7)
          Q3(I1)=ANS(4)-ANS(2)
      GOTO 998
C
C Next few lines are for a symmetrical R
999       THET(I1)=-1.0
          Q1(I1)=SQRT((ANS(1)+1.0)/2.0)
          Q2(I1)=SQRT((ANS(5)+1.0)/2.0)
          Q3(I1)=SQRT((ANS(9)+1.0)/2.0)
C
998       CALL MAP_UTIL_NORM(Q1(I1),Q2(I1),Q3(I1))
          THET(I1)=(THET(I1)-1.0)/2.0
C Convert THETA back to degrees
          THET(I1)=ACOS(THET(I1))*180.0/PI
C
1     CONTINUE
C
        RETURN
        END
